@@ -17,8 +17,10 @@ from sklearn.linear_model import SGDClassifier
 
 # import some data to play with
 iris = datasets.load_iris()
-X = iris.data[:, :2]  # we only take the first two features. We could
-                      # avoid this ugly slicing by using a two-dim dataset
+
+# we only take the first two features. We could
+# avoid this ugly slicing by using a two-dim dataset
+X = iris.data[:, :2]
 y = iris.target
 colors = "bry"
 
@@ -36,7 +38,7 @@ X = (X - mean) / std
 
 h = .02  # step size in the mesh
 
-clf = SGDClassifier(alpha=0.001, n_iter=100).fit(X, y)
+clf = SGDClassifier(alpha=0.001, max_iter=100, tol=1e-3).fit(X, y)
 
 # create a mesh to plot in
 x_min, x_max = X[:, 0].min() - 1, X[:, 0].max() + 1
@@ -56,7 +58,7 @@ plt.axis('tight')
 for i, color in zip(clf.classes_, colors):
     idx = np.where(y == i)
     plt.scatter(X[idx, 0], X[idx, 1], c=color, label=iris.target_names[i],
-                cmap=plt.cm.Paired)
+                cmap=plt.cm.Paired, edgecolor='black', s=20)
 plt.title("Decision surface of multi-class SGD")
 plt.axis('tight')
 
@@ -74,6 +76,7 @@ def plot_hyperplane(c, color):
     plt.plot([xmin, xmax], [line(xmin), line(xmax)],
              ls="--", color=color)
 
+
 for i, color in zip(clf.classes_, colors):
     plot_hyperplane(i, color)
 plt.legend()
