@@ -1,14 +1,16 @@
 """Testing for the boost module (sklearn.ensemble.boost)."""
 
+import pytest
 import numpy as np
+
 from sklearn.utils.testing import assert_array_equal, assert_array_less
 from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_equal, assert_true
+from sklearn.utils.testing import assert_equal, assert_greater
 from sklearn.utils.testing import assert_raises, assert_raises_regexp
 
 from sklearn.base import BaseEstimator
-from sklearn.cross_validation import train_test_split
-from sklearn.grid_search import GridSearchCV
+from sklearn.model_selection import train_test_split
+from sklearn.model_selection import GridSearchCV
 from sklearn.ensemble import AdaBoostClassifier
 from sklearn.ensemble import AdaBoostRegressor
 from sklearn.ensemble import weight_boosting
@@ -57,7 +59,7 @@ def test_samme_proba():
 
     # _samme_proba calls estimator.predict_proba.
     # Make a mock object so I can control what gets returned.
-    class MockEstimator(object):
+    class MockEstimator:
         def predict_proba(self, X):
             assert_array_equal(X.shape, probs.shape)
             return probs
@@ -66,7 +68,7 @@ def test_samme_proba():
     samme_proba = weight_boosting._samme_proba(mock, 3, np.ones_like(probs))
 
     assert_array_equal(samme_proba.shape, probs.shape)
-    assert_true(np.isfinite(samme_proba).all())
+    assert np.isfinite(samme_proba).all()
 
     # Make sure that the correct elements come out as smallest --
     # `_samme_proba` should preserve the ordering in each example.
@@ -74,6 +76,15 @@ def test_samme_proba():
     assert_array_equal(np.argmax(samme_proba, axis=1), [0, 1, 1, 1])
 
 
+def test_oneclass_adaboost_proba():
+    # Test predict_proba robustness for one class label input.
+    # In response to issue #7501
+    # https://github.com/scikit-learn/scikit-learn/issues/7501
+    y_t = np.ones(len(X))
+    clf = AdaBoostClassifier().fit(X, y_t)
+    assert_array_almost_equal(clf.predict_proba(X), np.ones((len(X), 1)))
+
+
 def test_classification_toy():
     # Check classification on a toy dataset.
     for alg in ['SAMME', 'SAMME.R']:
@@ -113,6 +124,12 @@ def test_iris():
         assert score > 0.9, "Failed with algorithm %s and score = %f" % \
             (alg, score)
 
+        # Check we used multiple estimators
+        assert_greater(len(clf.estimators_), 1)
+        # Check for distinct random states (see issue #7408)
+        assert_equal(len(set(est.random_state for est in clf.estimators_)),
+                     len(clf.estimators_))
+
     # Somewhat hacky regression test: prior to
     # ae7adc880d624615a34bafdb1d75ef67051b8200,
     # predict_proba returned SAMME.R values for SAMME.
@@ -123,11 +140,17 @@ def test_iris():
 
 def test_boston():
     # Check consistency on dataset boston house prices.
-    clf = AdaBoostRegressor(random_state=0)
-    clf.fit(boston.data, boston.target)
-    score = clf.score(boston.data, boston.target)
+    reg = AdaBoostRegressor(random_state=0)
+    reg.fit(boston.data, boston.target)
+    score = reg.score(boston.data, boston.target)
     assert score > 0.85
 
+    # Check we used multiple estimators
+    assert len(reg.estimators_) > 1
+    # Check for distinct random states (see issue #7408)
+    assert_equal(len(set(est.random_state for est in reg.estimators_)),
+                 len(reg.estimators_))
+
 
 def test_staged_predict():
     # Check staged predictions.
@@ -173,6 +196,8 @@ def test_staged_predict():
     assert_array_almost_equal(score, staged_scores[-1])
 
 
+@pytest.mark.filterwarnings('ignore: The default of the `iid`')  # 0.22
+@pytest.mark.filterwarnings('ignore: The default value of cv')  # 0.22
 def test_gridsearch():
     # Check that base trees can be grid-searched.
     # AdaBoost classification
@@ -256,43 +281,38 @@ def test_error():
                   X, y_class, sample_weight=np.asarray([-1]))
 
 
+@pytest.mark.filterwarnings('ignore:The default value of n_estimators')
 def test_base_estimator():
     # Test different base estimators.
     from sklearn.ensemble import RandomForestClassifier
-    from sklearn.svm import SVC
 
     # XXX doesn't work with y_class because RF doesn't support classes_
     # Shouldn't AdaBoost run a LabelBinarizer?
     clf = AdaBoostClassifier(RandomForestClassifier())
     clf.fit(X, y_regr)
 
-    clf = AdaBoostClassifier(SVC(), algorithm="SAMME")
+    clf = AdaBoostClassifier(SVC(gamma="scale"), algorithm="SAMME")
     clf.fit(X, y_class)
 
     from sklearn.ensemble import RandomForestRegressor
-    from sklearn.svm import SVR
 
     clf = AdaBoostRegressor(RandomForestRegressor(), random_state=0)
     clf.fit(X, y_regr)
 
-    clf = AdaBoostRegressor(SVR(), random_state=0)
+    clf = AdaBoostRegressor(SVR(gamma='scale'), random_state=0)
     clf.fit(X, y_regr)
 
     # Check that an empty discrete ensemble fails in fit, not predict.
     X_fail = [[1, 1], [1, 1], [1, 1], [1, 1]]
     y_fail = ["foo", "bar", 1, 2]
-    clf = AdaBoostClassifier(SVC(), algorithm="SAMME")
+    clf = AdaBoostClassifier(SVC(gamma="scale"), algorithm="SAMME")
     assert_raises_regexp(ValueError, "worse than random",
                          clf.fit, X_fail, y_fail)
 
 
 def test_sample_weight_missing():
-    from sklearn.linear_model import LogisticRegression
     from sklearn.cluster import KMeans
 
-    clf = AdaBoostClassifier(LogisticRegression(), algorithm="SAMME")
-    assert_raises(ValueError, clf.fit, X, y_regr)
-
     clf = AdaBoostClassifier(KMeans(), algorithm="SAMME")
     assert_raises(ValueError, clf.fit, X, y_regr)
 
@@ -308,7 +328,7 @@ def test_sparse_classification():
 
         def fit(self, X, y, sample_weight=None):
             """Modification on fit caries data type for later verification."""
-            super(CustomSVC, self).fit(X, y, sample_weight=sample_weight)
+            super().fit(X, y, sample_weight=sample_weight)
             self.data_type_ = type(X)
             return self
 
@@ -327,14 +347,14 @@ def test_sparse_classification():
 
         # Trained on sparse format
         sparse_classifier = AdaBoostClassifier(
-            base_estimator=CustomSVC(probability=True),
+            base_estimator=CustomSVC(gamma='scale', probability=True),
             random_state=1,
             algorithm="SAMME"
         ).fit(X_train_sparse, y_train)
 
         # Trained on dense format
         dense_classifier = AdaBoostClassifier(
-            base_estimator=CustomSVC(probability=True),
+            base_estimator=CustomSVC(gamma='scale', probability=True),
             random_state=1,
             algorithm="SAMME"
         ).fit(X_train, y_train)
@@ -347,29 +367,29 @@ def test_sparse_classification():
         # decision_function
         sparse_results = sparse_classifier.decision_function(X_test_sparse)
         dense_results = dense_classifier.decision_function(X_test)
-        assert_array_equal(sparse_results, dense_results)
+        assert_array_almost_equal(sparse_results, dense_results)
 
         # predict_log_proba
         sparse_results = sparse_classifier.predict_log_proba(X_test_sparse)
         dense_results = dense_classifier.predict_log_proba(X_test)
-        assert_array_equal(sparse_results, dense_results)
+        assert_array_almost_equal(sparse_results, dense_results)
 
         # predict_proba
         sparse_results = sparse_classifier.predict_proba(X_test_sparse)
         dense_results = dense_classifier.predict_proba(X_test)
-        assert_array_equal(sparse_results, dense_results)
+        assert_array_almost_equal(sparse_results, dense_results)
 
         # score
         sparse_results = sparse_classifier.score(X_test_sparse, y_test)
         dense_results = dense_classifier.score(X_test, y_test)
-        assert_array_equal(sparse_results, dense_results)
+        assert_array_almost_equal(sparse_results, dense_results)
 
         # staged_decision_function
         sparse_results = sparse_classifier.staged_decision_function(
             X_test_sparse)
         dense_results = dense_classifier.staged_decision_function(X_test)
         for sprase_res, dense_res in zip(sparse_results, dense_results):
-            assert_array_equal(sprase_res, dense_res)
+            assert_array_almost_equal(sprase_res, dense_res)
 
         # staged_predict
         sparse_results = sparse_classifier.staged_predict(X_test_sparse)
@@ -381,7 +401,7 @@ def test_sparse_classification():
         sparse_results = sparse_classifier.staged_predict_proba(X_test_sparse)
         dense_results = dense_classifier.staged_predict_proba(X_test)
         for sprase_res, dense_res in zip(sparse_results, dense_results):
-            assert_array_equal(sprase_res, dense_res)
+            assert_array_almost_equal(sprase_res, dense_res)
 
         # staged_score
         sparse_results = sparse_classifier.staged_score(X_test_sparse,
@@ -405,7 +425,7 @@ def test_sparse_regression():
 
         def fit(self, X, y, sample_weight=None):
             """Modification on fit caries data type for later verification."""
-            super(CustomSVR, self).fit(X, y, sample_weight=sample_weight)
+            super().fit(X, y, sample_weight=sample_weight)
             self.data_type_ = type(X)
             return self
 
@@ -421,26 +441,26 @@ def test_sparse_regression():
 
         # Trained on sparse format
         sparse_classifier = AdaBoostRegressor(
-            base_estimator=CustomSVR(),
+            base_estimator=CustomSVR(gamma='scale'),
             random_state=1
         ).fit(X_train_sparse, y_train)
 
         # Trained on dense format
         dense_classifier = dense_results = AdaBoostRegressor(
-            base_estimator=CustomSVR(),
+            base_estimator=CustomSVR(gamma='scale'),
             random_state=1
         ).fit(X_train, y_train)
 
         # predict
         sparse_results = sparse_classifier.predict(X_test_sparse)
         dense_results = dense_classifier.predict(X_test)
-        assert_array_equal(sparse_results, dense_results)
+        assert_array_almost_equal(sparse_results, dense_results)
 
         # staged_predict
         sparse_results = sparse_classifier.staged_predict(X_test_sparse)
         dense_results = dense_classifier.staged_predict(X_test)
         for sprase_res, dense_res in zip(sparse_results, dense_results):
-            assert_array_equal(sprase_res, dense_res)
+            assert_array_almost_equal(sprase_res, dense_res)
 
         types = [i.data_type_ for i in sparse_classifier.estimators_]
 
@@ -451,7 +471,6 @@ def test_sparse_regression():
 def test_sample_weight_adaboost_regressor():
     """
     AdaBoostRegressor should work without sample_weights in the base estimator
-
     The random weighted sampling is done internally in the _boost method in
     AdaBoostRegressor.
     """
@@ -466,3 +485,27 @@ def test_sample_weight_adaboost_regressor():
     boost = AdaBoostRegressor(DummyEstimator(), n_estimators=3)
     boost.fit(X, y_regr)
     assert_equal(len(boost.estimator_weights_), len(boost.estimator_errors_))
+
+
+def test_multidimensional_X():
+    """
+    Check that the AdaBoost estimators can work with n-dimensional
+    data matrix
+    """
+
+    from sklearn.dummy import DummyClassifier, DummyRegressor
+
+    rng = np.random.RandomState(0)
+
+    X = rng.randn(50, 3, 3)
+    yc = rng.choice([0, 1], 50)
+    yr = rng.randn(50)
+
+    boost = AdaBoostClassifier(DummyClassifier(strategy='most_frequent'))
+    boost.fit(X, yc)
+    boost.predict(X)
+    boost.predict_proba(X)
+
+    boost = AdaBoostRegressor(DummyRegressor())
+    boost.fit(X, yr)
+    boost.predict(X)
